local Fruit = require(
    (...):gsub("box", "fruit")
)

local images = IMG_Boxes
local broken_quads = {
    lg.newQuad(0, 0, 28, 24, images[1].broken:getDimensions()),
    lg.newQuad(28, 0, 28, 24, images[1].broken:getDimensions()),
    lg.newQuad(56, 0, 28, 24, images[1].broken:getDimensions()),
    lg.newQuad(84, 0, 28, 24, images[1].broken:getDimensions())
}

local Box = Class {

    TYPE = "box";
    images = images;
    broken_quads = broken_quads;

    init = function (self, level, collider, fruit_num, entities)

        self.box_type = math.random(#self.images)

        -- Initialisation des animations du l'ennemi --

        self.anims = {}
        self.images_anim = self.images[self.box_type]

        for k, image in pairs(self.images_anim) do
            if k ~= "broken" then
                local image_w = image:getWidth()
                local image_h = image:getHeight()
                local num_frame = image_w/28
                local g_anim = Anim8.newGrid(28, 24, image_w, image_h)
                self.anims[k] = Anim8.newAnimation(g_anim("1-"..num_frame,1), 0.075)
                if k == "hit" then self.hit_frame = num_frame end
            end
        end

        -- Initialisation du systeme de particule (break)

        self.psys = {}
        for i = 1, 4 do
            local psys = lg.newParticleSystem(self.images_anim.broken, 3)
            psys:setColors(1, 1, 1, 1, 1, 1, 1, 0)
            psys:setLinearAcceleration(-200, 200, 200, 200)
            psys:setQuads(self.broken_quads[i])
            psys:setSpin(-math.pi,math.pi)
            psys:setParticleLifetime(1, 2)
            self.psys[i] = psys
        end

        self.box_w, self.box_h = 28, 24
        self.particles_active = false

        -- Initialisation de la position et de l'objet physique --

        self.x = collider[2]+8
        self.y = collider[3]+12
        self.w = 28
        self.h = 28
        self.o = 14

        self.collider = collider[1]
        self.collider:setType("static")

        -- Callback de collision avec le joueur --

        self.collider:setPreSolve(function(c1, c2, contact)
            if c1.collision_class == "boxes" then

                if c2.collision_class == "player" then

                    local px, py = c2:getPosition()

                    if px+8 > self.x-4 and px-8 < self.x+4 then

                        local p = c2:getObject()
                        local pvx, pvy = p:getVelocity()
                        local nx, ny = contact:getNormal()

                        if ny > 0 then -- Joueur tape sur le dessus
                            self.life = self.life + pvy*.005
                            p.vy, p.jumpNum = -3, 1
                        elseif ny < 0 then -- Joueur tape en dessous
                            self.life = self.life + pvy*.0025
                        end

                        Sounds:play("impact_box"..math.random(10),true)
                        self:setState("hit")

                    end

                end

            end
        end)

        -- Initiliastion des fruits --

        self.fruits = {}
        local world = entities.level.world

        for i = 1, fruit_num do

            local x = self.x-self.o
            local y = self.y-self.o

            local fruit_collider = world
                :newCircleCollider(
                    self.x,self.y-4,8
                )

            fruit_collider
                :setCollisionClass("fruits")

            local fruit_object = Fruit(
                entities.level.index,{
                    fruit_collider,
                    self.x-8, self.y-12, 8
                }
            )

            fruit_object:setColliderType(
                "dynamic", false
            )

            self.fruits[i] = {
                obj = fruit_object;
                vx = math.random(0,10)/10;
                vy = -math.random(0,10)/10;
            }

        end

        -- Misc values --

        self:setState("idle")
        self.entities = entities
        self.life = 2*self.box_type

    end;

    destroy = function (self)
        if not self.collider:isDestroyed() then
            for _, fruit in ipairs(self.fruits) do fruit.obj:destroy() end
            for _, psys in ipairs(self.psys) do psys:release() end
            self.collider:destroy()
            self.destroyed = true
        end
    end;

    setState = function (self, state)

        if state ~= "broken" then
            self.anim = self.anims[state]
            self.anim:gotoFrame(1)
            self.state = state

        elseif not self.particles_active then

            self.particles_active = true
            self.collider:setActive(false)

            for i, psys in ipairs(self.psys) do
                for _ = 1, 3 do
                    psys:setPosition(self.x + math.random(self.box_w), self.y + math.random(self.box_h))
                    psys:setDirection(math.random() * math.pi * 2)
                    psys:emit(1)
                end
            end

            local ents = self.entities
            for i, fruit in ipairs(self.fruits) do
                fruit.obj:setColliderType("dynamic", true)
                ents[#ents+1] = fruit.obj
            end

        end
    end;

    uFruitsAnim = function (self, dt)

        for i = #self.fruits, 1, -1 do

            local fruit = self.fruits[i]
            local obj = fruit.obj

            if fruit.vy>=3
            or obj.collected
            or obj.collider:enter("walls")
            or obj.collider:enter("ground")
            or obj.collider:enter("ceiling")
            or obj.collider:enter("plateforms") then

                obj:setColliderType("kinematic", true)
                obj:setLinearVelocity(0,0)

                table.remove(self.fruits, i)
                goto continue

            end

            obj:setLinearVelocity(
                fruit.vx*300,
                fruit.vy*600
            )

            fruit.vx = math.max(
                fruit.vx-dt, 0
            )

            fruit.vy = fruit.vy + dt

            ::continue::

        end

    end;

    update = function (self, dt)

        -- Update timer state and state --

        if not self.particles_active then

            if self.state == "hit" then
                if self.anim.position == self.hit_frame then
                    if self.life <= 0 then
                        Sounds:play("box_break",true)
                        self:setState("broken")
                    else
                        self:setState("idle")
                    end
                end
            end

        else    -- Update particles systeme (borken mode) --

            self:uFruitsAnim(dt)

            if #self.fruits == 0 then
                self:destroy()
                return true
            end

            for i, psys in ipairs(self.psys) do
                psys:update(dt)
            end

        end

        -- Update anim --

        self.anim:update(dt)

    end;

    draw = function (self)

        if not self.particles_active then
            self.anim:draw(
                self.images_anim[self.state],
                self.x, self.y,0,1,1,
                self.o, self.o
            )
        elseif not self.destroyed then
            for _, psys in ipairs(self.psys) do
                lg.draw(psys)
            end
        end

    end;

}

return Box