-- Collider: t1, t2 == 32x32 | t3 == 16x16

local Rock = Class {

    TYPE = "enemy";
    images = IMG_Enemies.rocks;

    sizes = {
        {w = 38, h = 34, offset_y = 0};
        {w = 32, h = 28, offset_y = 2};
        {w = 22, h = 18, offset_y = 2};
    };

    init = function (self, level, collider)

        self.world = level.world

        -- get type --

        local _type = collider[1].type
        local _ntype = tonumber(_type)

        -- Initialisation de l'objet physique --

        self.entities = {
            self:createEntity(
                collider[1],
                collider[2],
                collider[3],
                1, _ntype
            )
        }

    end;

    destroy = function (self)
        for i = #self.entities, 1, -1 do
            self.entities[i]:destroy()
            table.remove(self.entities,i)
        end
    end;

    createEntity = function (self, collider, x, y, dir, _type)

        -- Create object and collider --

        local offset_y = self.sizes[_type].offset_y
        local w = self.sizes[_type].w
        local h = self.sizes[_type].h

        local object = {
            setState=self.setState,
            images=self.images,
            offset_y=offset_y,
            x=x,y=y, w=w,h=h,
            ox=w/2, oy=h/2,
            dir=dir
        }

        function object:destroy()
            if not object.collider:isDestroyed() then
                object.isDestroyed = true
                object.collider:destroy()
            end
        end

        if not collider then
            collider = self.world:newRectangleCollider(
                x, y, w, h
            )
        end

        collider:setObject(object)
        collider:setCollisionClass("enemies")

        collider:setPreSolve(function(c1, c2, contact)

            if not object.isDestroyed
            and c1.collision_class == "enemies" then

                local obj = collider:getObject()

                if c2.collision_class == "ground"
                or c2.collision_class == "plateforms" then

                    if obj.state == "run" then

                        local px, py = c2:getPosition()

                        if (obj.dir < 0 and obj.x-8 < (px-c2.w/2)+8)
                        or (obj.dir > 0 and obj.x+8 > (px+c2.w/2)-8)
                        then
                            if math.random() < .5 then
                                obj:setState("idle")
                            else
                                obj.dir = -obj.dir
                            end
                        end

                    end

                elseif c2.collision_class == 'player' then

                    local px, py = c2:getPosition()
                    local p = c2:getObject()

                    if py+p.o <= obj.y then
                        Sounds:play("on_enemy_bounce", true)
                        p.vy, p.jumpNum = -3, 1
                        obj:setState("hit")
                    else
                        p:kill()
                    end

                elseif c2.collision_class == "walls" then
                    local wall_x = c2:getPosition()
                    if wall_x < obj.x and obj.dir < 0
                    or wall_x > obj.x and obj.dir > 0 then
                        obj.dir = -obj.dir
                    end

                elseif c2.collision_class == "fruits" then
                    contact:setEnabled(false)

                elseif c2.collision_class == "enemies" then
                    local o_obj = c2:getObject()
                    if o_obj and o_obj.shell_move then
                        obj:setState("hit")
                        o_obj:hitOther()
                    else
                        contact:setEnabled(false)
                    end
                end

            end

        end)

        -- Create an animation list --

        object.anims = {}

        for k, image in pairs(self.images) do

            local s = self.sizes[
                tonumber(k:match("[123]"))
            ]

            local image_w = image:getWidth()
            local image_h = image:getHeight()
            local num_frame = image_w/s.w

            local g_anim = Anim8.newGrid(s.w, s.h, image_w, image_h)
            object.anims[k] = Anim8.newAnimation(g_anim("1-"..num_frame,1), 0.05)

        end

        -- Init misc values --

        object.type = _type
        object.collider = collider
        object.life = self.permuteLifeType(_type)
        object:setState("idle")

        return object

    end;

    permuteLifeType = function (n)
        return math.clamp(math.abs(n-4),1,3)
    end;

    entityHit = function (self, entity)

        local life = entity.life-1
        local x = entity.x
        local y = entity.y
        local ox = entity.ox

        for i = #self.entities, 1, -1 do
            if self.entities[i] == entity then
                table.remove(self.entities, i)
                entity:destroy(); break
            end
        end

        if life >= 1 then
            self.entities[#self.entities+1] = self:createEntity(
                nil, x-ox, y, math.random()<.5 and -1 or 1, self.permuteLifeType(life)
            )
            self.entities[#self.entities+1] = self:createEntity(
                nil, x+ox, y, math.random()<.5 and -1 or 1, self.permuteLifeType(life)
            )
        end

    end;

    setState = function (self, state)

        if state ~= self.state then

            local key = "rock"..self.type.."_"..state

            self.image = self.images[key]
            self.anim = self.anims[key]
            self.anim:gotoFrame(1)
            self.timer_state = 0
            self.state = state

            if self.type == 3
            and state == "hit" then
                self:destroy()
                self.vy = -1
            end

        end

    end;

    update = function (self, dt)

        for i = #self.entities, 1, -1 do

            local e = self.entities[i]
            local state = e.state

            -- Update position --

            if state ~= "hit" then
                e.x = e.collider:getX()
                e.y = e.collider:getY()+e.offset_y
            end

            if e.y-e.oy > GH then
                self:entityHit(e)
                goto continue
            end

            -- Update timer state and state --

            e.timer_state = e.timer_state + dt

            if state == "idle" then
                if e.timer_state >= 1 then
                    e:setState("run")
                    e.dir = -e.dir
                end

            elseif state == "run" then
                e.collider:setLinearVelocity(200*e.dir, 0)
                if e.timer_state >= 2 then e:setState("idle") end

            elseif state == "hit" then

                if e.type == 3 then
                    e.y = e.y + e.vy * 300 * dt
                    e.vy = e.vy + 5 * dt
                elseif e.timer_state >= .1 then
                    self:entityHit(e)
                    goto continue
                end

            end

            -- Update anim --

            if not (
                state=="hit"
                and e.anim.position == 5
            ) then
                e.anim:update(dt)
            end

            ::continue::

        end

        return (#self.entities==0)

    end;

    draw = function (self)

        for i = #self.entities, 1, -1 do
            local e = self.entities[i]

            e.anim:draw(
                e.image,
                e.x,
                e.y,0,
                -e.dir,1,
                e.ox,
                e.oy
            )

        end

    end;

}

return Rock